import krieger.oclsolve.*;

public class Example {	
	public static void main(String[] args) {
		// We do not define any classes in this example.
		// (We stick to the primitive types and the OCL standard library types.)
		ClassTranslatorBase<Class<?>, String> classTranslator = new ClassTranslatorBase<Class<?>, String>();
		
		// We use 32 bits for integers (the "String" type parameter denotes a type used for identifying classes).
		ClassifierFactory<String> classifierFactory = classTranslator.createClassifierFactory(32);
		
		// We obtain an object representing the type of plain integers.
		// The multiplicity passed indicates that null values are allowed. 
		Classifier<?> intType = classifierFactory.getIntegerType(new Multiplicity(0,1,false,false));
		
		// We obtain an object representing the type of integer sets.
		// The multiplicity passed describes an unordered collection of unbounded size containing unique elements. 
		Classifier<?> intSet = classifierFactory.getIntegerType(new Multiplicity(0,-1,false,true));
		
		// We define variables for holding the set, the subset, and the value of the subset sum.
		Constant<?> set = intSet.newConstant();
		Variable<?> subset = intSet.newVariable();
		Constant<?> n = intType.newConstant();
		
		// We start defining the constraint satisfaction problem.
		Problem problem = new Problem();
		SetExpression setExpr = (SetExpression) set.getExpression();
		SetExpression subsetExpr = (SetExpression) subset.getExpression();
		IntegerExpression nExpr = (IntegerExpression) n.getExpression();
		
		// The set must include all elements of the subset.
		// In OCL: set->includesAll(subset)
		problem.addConstraint(setExpr.includesAll(subsetExpr));
		
		// The subset sum must not be larger than prescribed.
		// In OCL: subset->sum() <= n
		problem.addConstraint(subsetExpr.sum().leq(nExpr));
		// Replace "leq" by "oclIsEqualTo" to force equality
		
		// We choose to maximize the sum of the subset as objective function.
		problem.maximize(subsetExpr.sum());
		
		// We assign values to the constants defined.
		set.assignValue(new int[] {1, 2, 8});
		n.assignValue(4);
		
		// Finally we solve the constraint satisfaction problem.
		classifierFactory.addToProblem(problem);
		set.addToProblem(problem);
		subset.addToProblem(problem);
		n.addToProblem(problem);
		
		boolean solved = problem.solve();
		System.out.println("Solution found: " + solved);
		
		System.out.println("Satisfying subset: " + subsetExpr.evaluate());
	}
}